# -*- coding: utf-8 -*-
#
# File: tests/testMailServices.py
#
# Copyright (c) 2007 atReal
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """atReal <contact@atreal.net> : Matthias Broquet (tiazma)"""
__docformat__ = 'plaintext'
__licence__ = 'GPL'

# First, above all, execute the framework.py

#import os, sys 
#if __name__ == '__main__':
#  execfile(os.path.join(sys.path[0],'framework.py'))

# Install the PloneLanguageTool Product in Zope

#from Testing import ZopeTestCase
#ZopeTestCase.installProduct('ARMailServices')

# Initialize our Plone and default install PloneLanguageTool in it

from Products.PloneTestCase import PloneTestCase
from Products.CMFCore.utils import getToolByName
PloneTestCase.installProduct('ARMailServices')
PloneTestCase.setupPloneSite()


class TestMailServices (PloneTestCase.PloneTestCase): 
  """
    A class to test MailServices
  """

  def addUser (self,id,passwd="secret",gp=('Reviewers',),email="contact@atreal.net",fn="john doe"):
    pms = self.portal.portal_membership
    pms.addMember(id, passwd, [], [])
    member = pms.getMemberById(id)
    member.setMemberProperties({'fullname': fn, 'email': email,})
    self.portal.acl_users.userSetGroups(id, groupnames=gp)


  def addTool (self):
    self.addProduct('ARMailServices')
    return getToolByName(self.portal, 'armail_tool')


  def testGetAllMembersInfos (self):
    """ 
      If I create a new user, his informations must be returned
    """
    self.addUser('tommy')
    amt = self.addTool()
    l = amt.getAllMembersInfos()
    # We have to get 2 items in l : default_user and tommy !!
    self.failUnlessEqual(len(l),2)
    # we have to find tommy !
    self.failUnless("tommy" in [item[0] for item in l])


  def testGetEmailsByMemberIds(self):
    self.addUser('tommy')
    amt = self.addTool()
    l = amt._getEmailsByMemberIds(['tommy'])
    self.failUnlessEqual(len(l),1)
    self.failUnless("contact@atreal.net" in l)


  def testGetEmailsByGroups(self):
    for user in ['raymond','alfred','gerard']:
      self.addUser(user,gp=('Administrators','Reviewers'),email='contact@atreal.net')
    amt = self.addTool()
    l = amt._getEmailsByGroups(('Reviewers',))
    # We have to get 3 emails, all equivalent to contact@atreal.net
    self.failUnlessEqual(len(l),3)
    for email in l:
      self.failUnlessEqual(email,'contact@atreal.net')
    # Even if the 3 guys are in both groups, we just have 3 results
    l = amt._getEmailsByGroups(('Reviewers','Administrators'))
    self.failUnlessEqual(len(l),3)
    for email in l:
      self.failUnlessEqual(email,'contact@atreal.net')


# Now we need our testcase inside a test suite.

def test_suite():
  from unittest import TestSuite, makeSuite
  suite = TestSuite()
  suite.addTest(makeSuite(TestMailServices))
  return suite

# and if you want to be able to run your suite directly (python testDocument.py)

if __name__ == '__main__':
  framework()

