# -*- coding: utf-8 -*-
#
# File: tool/ARMail_Tool.py
#
# Copyright (c) 2006 atReal
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """atReal <contact@atreal.net> : Matthias Broquet (tiazma)"""
__docformat__ = 'plaintext'
__licence__ = 'GPL'

from Products.CMFCore.utils import UniqueObject
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass
from Products.CMFCore.utils import getToolByName
from Products.ARMailServices.interface import IARMsTool

from AccessControl import Unauthorized
from zExceptions import NotFound

from AccessControl import ClassSecurityInfo

from Products.ARMailServices.config import *
from zope.interface import implements

class ARMailTool (UniqueObject, SimpleItem):
  """ ARMailTool  .... """
  id = 'armail_tool'
  meta_type= 'ARMail Tool'
  plone_tool = 1

  implements(IARMsTool)
  
  security = ClassSecurityInfo()
  
  security.declarePublic('getAllMembersInfos')
  def getAllMembersInfos (self):
    """ Return a list of tuple (user_id,fullname,email) for all the members
        of the portal
    """
    infos = []
    pmt = getToolByName(self, 'portal_membership')
    infos = [(member.id,member.getProperty('fullname',None),member.getProperty('email',None)) for member in pmt.listMembers()]
    return infos
    

  security.declarePublic('globalSendMail')
  def globalSendMail (self,request,info):
    """ Mail all targeted users
    """
    session=request.SESSION
    kt = {'group_to':set(),
          'group_cc':set(),
          'user_to':set(),
          'user_cc':set(),
          'email_to':set([item for item in request.get('email_to','').replace(' ','').split(';') if len(item)]),
          'email_cc':set([item for item in request.get('email_cc','').replace(' ','').split(';') if len(item)]),
         }
    # we load all the email in set
    for k in kt.keys():
      if k.startswith('g') and len(session[k]):
        kt[k]=set(self._getEmailsByGroups(session[k]))
      elif k.startswith('u') and len(session[k]):
        kt[k]=set(self._getEmailsByMemberIds(session[k]))
    
    # we join all this stuff to avoid multisending
    to_mails = list(kt['group_to'] | kt['user_to'] | kt['email_to'])
    cc_mails = list(kt['group_cc'] | kt['user_cc'] | kt['email_cc'])
    mhost,addinf = self._loadAdditionnalInfos()
    info.update(addinf)
    result = self._sendMail(mhost,info,to_mails,cc_mails)
    # we store the origine url if it exists and  empty the session vars
    url = session.get('isurl',False)
    session.clear()
    return url
    

  security.declarePrivate('_getEmailsByMemberIds')
  def _getEmailsByMemberIds(self,users_ids):
    """ Return the emails of all members
    """
    pmt = getToolByName(self, 'portal_membership')
    emails = [pmt.getMemberById(id).getProperty('email',None) for id in users_ids]
    return emails


  security.declarePrivate('_getEmailsByGroups')
  def _getEmailsByGroups(self,groups_ids):
    """ Return the emails of all members of groups
    """
    pmt = getToolByName(self, 'portal_membership')
    result=[]
    idsYetListed=[]
    for group in groups_ids:
      emails=[]
      members = pmt.searchForMembers(groupname=group)
      for member in members:
        if member.id not in idsYetListed:
          idsYetListed.append(member.id)
          emails.append(member.getProperty('email',None))
      result.extend(emails) 
    return result
    

  security.declarePrivate('_buildResult')
  def _buildResult(self,result):
    """ Return a tuple
    """
    for item in result:
      for email,exc in item.items():
        print "Send mail to user identified by %s failed with the error %s." \
              % (email,exc) 


  security.declarePrivate('_loadAdditionnalsInfos')
  def _loadAdditionnalInfos (self):
    """ Return a tuple, with mail host object and a dict, composed of
        some additionnal infos, like encoding and envelope_from.
    """
    plone_utils = getToolByName(self, 'plone_utils')
    site_properties = getToolByName(self, 'portal_properties').site_properties
    mhost = getToolByName(self, 'MailHost')
    info = {'envelope_from' : site_properties.email_from_address,
            'encoding' : plone_utils.getSiteEncoding(),
           }
    return (mhost,info)
    

  security.declarePrivate('_sendMail')
  def _sendMail(self, host, info, email_to, email_cc=None):
    """ Send an email to all emails from emailsList,
        with the info passsed in info, via the host object.
        Return a dict with the (email,exception) of problematics 
        emails, or empty if no errors occured.
    """
    result = {}
    try:
      host.secureSend(info['body'], email_to,
                     info['envelope_from'], subject=info['subject'],
                     mcc=email_cc,
                     subtype='plain', charset=info['encoding'],
                     debug=False, From=info['send_from_address'])
    except Exception,e:
      result['email']=e.__class__.__name__+' : '+str(e)
    #print 'errors on _sendMail : '+ str(result)
    return result


  security.declarePublic('formToSession')
  def formToSession(self, context):
    """ Get recipient's list from request and store it
        into session variables 
    """
    request = context.REQUEST
    import re
    target = {'group_to':[],
              'group_cc':[],
              'user_to':[],
              'user_cc':[],
             }
    rex = {'group_to':re.compile("^tm_gp_to"),
           'group_cc':re.compile("^tm_gp_cc"),
           'user_to':re.compile("^tm_us_to"),
           'user_cc':re.compile("^tm_us_cc"),
          }
    chbx_exist = False

    for k in request.keys():
      for t,v in rex.items():
        if re.match(v,k):
          chbx_exist=True
          target[t].append(re.sub(v,'',k))

    if not chbx_exist:
      return False
    else:
      # if we chose to send a document's reference :
      isurl = request.get('isurl',False)
      if isurl:
        site_properties = getToolByName(self, 'portal_properties').site_properties
        request.SESSION['message'] = 'Publication of new document on %s : %s' \
             % (site_properties.title,context.TitleOrId()) \
            + '\nURL : %s\n' % (isurl) + request.get('body','')
        request.SESSION['isurl'] = isurl
      # optionally added : add the sender to recipient's list
      target['user_to'].append(str(request['AUTHENTICATED_USER']))
      for k,v in target.items():
        request.SESSION[k] = v
      return True

 
InitializeClass(ARMailTool)
